@@ -1,23 +1,19 @@
 """Unit tests for extraction plugin"""
 
-from Products.PloneTestCase import PloneTestCase
-from Products.WebServerAuth.plugin import usernameKey, defaultUsernameHeader, stripDomainNamesKey, stripWindowsDomainKey, usernameHeaderKey
+from Products.WebServerAuth.plugin import usernameKey, defaultUsernameHeader, stripDomainNamesKey, stripWindowsDomainKey, usernameHeaderKey, cookieNameKey, cookieCheckEnabledKey
 from Products.WebServerAuth.tests.base import WebServerAuthTestCase
 
-
-PloneTestCase.installProduct('WebServerAuth')
-PloneTestCase.setupPloneSite(products=['WebServerAuth'])
-
-
 _username = 'someUsername'
 _domain = 'example.com'
 _userAtDomain = '%s@%s' % (_username, _domain)
 _userWinDomain = 'EXAMPLE\%s' % _username
 
+
 class _MockRequest(object):
     def __init__(self, environ=None):
         self.environ = environ or {}
 
+
 class TestExtraction(WebServerAuthTestCase):
     def afterSetUp(self):
         self.plugin = self._plugin()
@@ -38,22 +34,28 @@ class TestExtraction(WebServerAuthTestCase):
         """Assert the name of the header in which the username is passed can be changed."""
         alternateHeader = 'HTTP_REMOTE_USER'
         request = _MockRequest(environ={alternateHeader: _username})
-        saveHeader = self.plugin.config[usernameHeaderKey]
         self.plugin.config[usernameHeaderKey] = alternateHeader
-        try:
-            self.failUnlessEqual(self.plugin.extractCredentials(request), {usernameKey: _username})
-        finally:
-            self.plugin.config[usernameHeaderKey] = saveHeader
+        self.failUnlessEqual(self.plugin.extractCredentials(request), {usernameKey: _username})
+    
+    def testCookieCheck(self):
+        """If configured to look for a cookie, check that we don't extract
+        crentials if the cookie is missing."""
+        
+        request = _MockRequest(environ={defaultUsernameHeader: _username})
+        request.cookies = {}
+        
+        self.plugin.config[cookieNameKey] = 'foo'
+        self.plugin.config[cookieCheckEnabledKey] = True
+        
+        self.failUnless(self.plugin.extractCredentials(request) is None, msg="Found credentials to extract, even though we shouldn't have.")
+        request.cookies['foo'] = 'bar'
+        self.failUnlessEqual(self.plugin.extractCredentials(request), {usernameKey: _username})
     
     def _testDomainStripping(self, configKey, configSetting, incomingUsername, outgoingUsername):
         """Set the `configKey` config setting to `configSetting`, and make sure the username `incomingUsername` is transformed to `outgoingUsername` upon extraction."""
         request = _MockRequest(environ={defaultUsernameHeader: incomingUsername})
-        saveStrip = self.plugin.config[configKey]
         self.plugin.config[configKey] = configSetting
-        try:
-            self.failUnlessEqual(self.plugin.extractCredentials(request), {usernameKey: outgoingUsername})
-        finally:
-            self.plugin.config[configKey] = saveStrip
+        self.failUnlessEqual(self.plugin.extractCredentials(request), {usernameKey: outgoingUsername})
     
     def testEmailDomainStripping(self):
         """Assert choosing to not strip the domain off the end of a whatever@here.com username works."""
